"""
This example demonstrates the proper use of project: digiplot
"""
import sys
import os

sys.path.insert(0, os.path.abspath("../../"))  # needed to find digiplot development version

from digiplot.digiplot import DigiPlot

# TODO: create working example of project digiplot

my_class = DigiPlot()

